#### Examples how to use GTM and GTM* from paper  "Assessing allele-specific expression across multiple tissues from RNA-seq read data"
#### Matti Pirinen 29-Dec-2014

#generate some data
nv=40 #number of variants
nt=sample(5:10,replace=TRUE,size=nv,prob=rep(1,6)) #number of tissues for each variant
nreads=20 #number of reads per each tissue
states=sample(c(1:5),size=nv,replace=TRUE,prob=c(1,0,0,1,0.5)) #true state for each variant

theta=c(0.5,0.25,0.01) #non ref allele freq for each group (1=NOASE ,2=MODASE or 3=SNGASE)

y.list=list()
true.groups=list()
for(set in 1:nv){
  if(states[set] %in% c(1:3)) groups=rep(states[set],nt[set])
  if(states[set]==4) groups=c(rep(1,3),rep(2,nt[set]-3))#HET0
  if(states[set]==5) groups=c(rep(3,3),rep(2,nt[set]-3))#HET1
  groups=sample(groups)
  true.groups[[set]]=groups
  y=rep(NA,nt[set])
  for(i in 1:nt[set]){y[i]=rbinom(1,size=nreads,prob=theta[groups[i]])}
  y=cbind(nreads-y,y)
  rownames(y)=paste("T",1:nt[set],sep="")
  y.list[[set]]=y
}

source("ASE_29Dec2014.R")
pr.beta=c(2000,2000,36,12,80,1)
#pr.intv=c(0.48,0.52,0.52,0.95,0.95,1) #truncate to intervals
pr.intv=rep(NA,6) #do not truncate
indp=FALSE #independent frequencies among tissues in same group?
two.sided=FALSE #two sided frequency distributions?
niter=100
burnin=10
prior.pi=rep(1,5)
group.distance=c(1,1,0.5)

res.list=list() #will be results from single variant analyses
posteriors=rep(0,6) #combined state probabilities from single var analyses
for(i in 1:nv){
  res.list[[i]]=gtm(y.list[[i]],pr.beta=pr.beta,pr.intv=pr.intv,niter=niter,burnin=burnin,two.sided=two.sided,independent=indp)
  posteriors=posteriors+as.numeric(res.list[[i]]$state.posteriors)
}
posteriors=posteriors/nv

#apply hierarchical model:
hm.res=gtm.star(y.list,pr.beta=pr.beta,pr.intv=pr.intv,pr.pi=prior.pi,niter=niter,burnin=burnin,two.sided=two.sided,independent=indp)


sing.res=matrix(NA,nrow=5,ncol=3) #matrix with 95% intervals for single variant analyses
sing.res[,1]=c(posteriors[1:5])#,sum(posteriors[4:5]))
sing.res[,2]=sing.res[,1]-1.96*sqrt(sing.res[,1]*(1-sing.res[,1])/nv);sing.res[sing.res[,2]<0,2]=0
sing.res[,3]=sing.res[,1]+1.96*sqrt(sing.res[,1]*(1-sing.res[,1])/nv)

plot(0,xlim=c(0,6),ylim=c(0,1),col="white",xlab="state",ylab="pi",xaxt="n")
axis(1,at=c(1:5),labels=c("NOASE","MODASE","SNGASE","HET0","HET1"))
for(i in 1:5){
  points(i-0.2,hm.res$prop.posteriors[i,1],pch=19,col="springgreen")
  points(i+0.2,sing.res[i,1],pch=19,col="blue")
  arrows(i-0.2,hm.res$prop.posteriors[i,2],i-0.2,hm.res$prop.posteriors[i,3],code=3,angle=90,length=0.1)
  arrows(i+0.2,sing.res[i,2],i+0.2,sing.res[i,3],code=3,angle=90,length=0.1)
  points(i,as.numeric((table(states)/nv)[as.character(i)]),pch="X",col="red")#plot true values
}


hm.res$prop.posteriors[,1] #state probabilities from hierarchical model
round(apply(matrix(unlist(hm.res$state.posteriors),byrow=TRUE,ncol=6),2,mean),2) #another version from hier model, 6th state=TISSUE SPECIFIC
round(posteriors,2) #single var analyses, 6th state=TISSUE SPECIFIC
table(states)/nv #true proportions




########################################




### Codes to create results for paper "Assessing allele specific expression across multiple tissues from RNA-seq read data"
### Contains code for Figures 1-4 + Supp Figure 3 and for exact calculation of state probabilities
### Needs some data files (www.iki.fi/mpirinen).


###########
FIGURE 1 BEGINS.
###########

#Priors
a0=b0=2000
a1=36;b1=12
a2=80;b2=1
#pdf(paste("Gibbs_priors_",a0,"-",b0,"_",a1,"-",b1,"_",a2,"-",b2,".pdf",sep=""),width=6,height=4,pointsize=7.5,paper="special")
#postscript(paste("Gibbs_priors_",a0,"-",b0,"_",a1,"-",b1,"_",a2,"-",b2,".eps",sep=""),width=6,height=4,pointsize=7.5,paper="special")
par(mar=c(4,6,1.2,1.5))
x=seq(0,1,length.out=1000)
plot(x,dbeta(x,a0,b0),t="l",main="",xlab=expression(paste("proportion (",theta,")",sep="")),ylab="density",xlim=c(0,1),ylim=c(0,50),xaxs="i",yaxs="i",xaxt="n",lty=3,cex.lab=1.8,cex.axis=1.6,cex.main=1.6,lwd=1.3)
axis(1,at=c(0,0.25,0.5,0.75,1),cex.axis=1.6)
lines(x,0.5*dbeta(x,a1,b1)+0.5*dbeta(x,b1,a1),lty=5,lwd=1.3)
lines(x,0.5*dbeta(x,a2,b2)+0.5*dbeta(x,b2,a2),lty=1,lwd=2)
text(0.05,10,"S",cex=2.5)
text(0.25,5,"M",cex=2.5)
text(0.54,10,"N",cex=2.5)
#dev.off()

qbeta(c(0.025,0.975),b1,a1)
qbeta(c(0.025,0.975),a0,b0)
qbeta(c(0.025,0.975),b2,a2)

###########
FIGURE 1 ENDS.
###########



###########
FIGURE 2 BEGINS.
###########

source("ASE_29Dec2014.R")
#Assess performance on scenarios 1-9 (Table 1):

#Data for each scenario is in file "../simu_res/single/3class_simu_res_nt_N_M_S_nreads_NREADS_2000-2000_36-12_80-1_niter_2000.Rdata"
#where N, M, and S are the number of tissues in each group, and NREADS is the number of reads per tissue (10 or 50)
#That file contains 'data.list', a list of simulated data set and
#res.list, a list of results from GTM (NOTE that what is config.posteriors here has been changed to state.posteriors) 


par.tag="2000-2000_36-12_80-1"
do.png=FALSE
#if(do.png) png("assessing_probabilities_9_sets.png",width=700,height=900)
#if(!do.png) pdf(paste("assessing_probabilities_9_sets_",par.tag,".pdf",sep=""),paper="special",width=3.5,height=7,pointsize=7)
#if(!do.png) postscript(paste("assessing_probabilities_9_sets_",par.tag,".eps",sep=""),paper="special",width=3.5,height=7,pointsize=7)

if(do.png) coeff=1 else coeff=0.8
nts=c(5,10,30)

cols=c("white","gray","black","cyan","violet")
fill.cols=c("white","gray","black","cyan","cyan","violet","violet")
legends=c("NOASE","MODASE","SNGASE","HET0","HET0","HET1","HET1")
legends.details=c("(ALL in N)","(ALL in M)","(ALL in S)","(1 in S ; T-1 in N)","(1 in N ; T-1 in S)","(1 in S ; T-1 in M)","(1 in M ; T-1 in S)","(T/2 in N ; T/2 in M)","(T/2 in M ; T/2 in S)")

if(do.png) layout(matrix(c(1:18),ncol=2,nrow=9,byrow=FALSE),height=c(1.3,rep(1,7),1.7),width=c(1.2,1))
if(!do.png) layout(matrix(c(1:18),ncol=2,nrow=9,byrow=FALSE),height=c(1.3,rep(1,7),1.9),width=c(1.2,1))

for(round in 1:2){
  nreads=c(10,50)[round]
  for(scen in c(1:9)){
    res=matrix(NA,nrow=5,ncol=length(nts))
    icol=1
    for(nt in nts){
      if(scen==1) load(paste("data/3class_simu_res_nt_",nt,"_0_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==2) load(paste("data/3class_simu_res_nt_0_",nt,"_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==3) load(paste("data/3class_simu_res_nt_0_0_",nt,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==4) load(paste("data/3class_simu_res_nt_",nt-1,"_0_1_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==5) load(paste("data/3class_simu_res_nt_1_0_",nt-1,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==6) load(paste("data/3class_simu_res_nt_0_",nt-1,"_1_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==7) load(paste("data/3class_simu_res_nt_0_1_",nt-1,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==8) load(paste("data/3class_simu_res_nt_",ceiling(nt/2),"_",floor(nt/2),"_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==9) load(paste("data/3class_simu_res_nt_0_",ceiling(nt/2),"_",floor(nt/2),"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
          
      a=logprior.distance(nt)
      prior.odds=c(rep(0.25,3),exp(c(a$log.sum.prior.h0,a$log.sum.prior.h1)))
      prior.odds=prior.odds/(1-prior.odds)
      nsets=length(data.list)
      chosen=rep(0,5)
      for(i in 1:nsets){
        p=as.numeric(res.list[[i]]$config.posteriors)
        posterior.odds=p/(1-p)
        chosen=chosen+p[1:5]
      }
      res[,icol]=chosen/nsets
      icol=icol+1
    }
    ord=rev(order(res[,1]))
    if(scen==9){
      if(round==1) {
        par(mar=c(8,4,3,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,xlab="",cex.lab=1.5,horiz=TRUE)
        if(do.png) {
          text(0.5,-1.2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.5)
          legend(0.1,-1.5,legend=c("NOASE"),fill=cols[1],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.4,-1.5,legend=c("MODASE"),fill=cols[2],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.7,-1.5,legend=c("SNGASE"),fill=cols[3],border="black",xpd=TRUE,bty="n",cex=1.5)
        }
        if(!do.png){
          text(0.5,-2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.4)
          legend(-0.2,-2.4,legend=c("NOASE"),fill=cols[1],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.2,-2.4,legend=c("MODASE"),fill=cols[2],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.6,-2.4,legend=c("SNGASE"),fill=cols[3],border="black",xpd=TRUE,bty="n",cex=1.4)
      
        }
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
      }else {
        par(mar=c(8,1,3,2))        
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,xlab="",cex.lab=1.5,horiz=TRUE)
        if(do.png) {
          text(0.5,-1.2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.5)
          legend(0.0,-1.5,legend=c("HET0"),fill=cols[4],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.3,-1.5,legend=c("HET1"),fill=cols[5],border="black",xpd=TRUE,bty="n",cex=1.5)
        }
        if(!do.png){
          text(0.5,-2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.4)
          legend(0.0,-2.4,legend=c("HET0"),fill=cols[4],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.4,-2.4,legend=c("HET1"),fill=cols[5],border="black",xpd=TRUE,bty="n",cex=1.4)
        }
      }
      axis(1,at=c(0,0.25,0.5,0.75,1),cex.axis=1.3*coeff)
    }
    if(scen>1 & scen < 9){
      if(round==1) {
        par(mar=c(0.1,4,3,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
      }else{
        par(mar=c(0.1,1,3,2))
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,horiz=TRUE)
      }
    }
    if(scen==1){
      if(round==1) {
        par(mar=c(0.1,4,6,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
        text(0.5,6,paste("Number of reads =",nreads),xpd=TRUE,cex=1.7)
      }else{
        par(mar=c(0.1,1,6,2))
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(0.5,6,paste("Number of reads =",nreads),xpd=TRUE,cex=1.7)
      }
      
    }
    if(round==1){
      if(do.png) text(0.65,4.2,paste("SCENARIO",scen),xpd=TRUE,cex=1.5)
      if(!do.png) text(0.65,4.2,paste("SCENARIO",scen),xpd=TRUE,cex=1.2)
    }else{
      if(do.png ) text(0.3,4.2,legends.details[scen],xpd=TRUE,cex=1.5)
      if(!do.png) text(0.5,4.2,legends.details[scen],xpd=TRUE,cex=1.2)
    }
    print(res)
  }
}


#dev.off()


###########
FIGURE 2 ENDS.
###########



###########
FIGURE 3 BEGINS.
###########



###Make distribution of Cochran's Q for p-value calculation
#This takes some time so results can be saved for further use (see 'save' below)
nt=30
niter=100000
nreads=rep(10,nt)

mus=seq(0.01,0.5,length.out=50)
mu.stats=list()

mu.i=1
for(mu in mus){
  
  theta=rep(mu,nt)
  st=rep(NA,niter)
  for(i in 1:niter){
    yy=rep(NA,nt)
    for( t in 1:nt){yy[t]=rbinom(1,prob=theta[t],size=nreads[t])}
    ind=which(yy==0)
    if(length(ind)>0) yy[ind]=yy[ind]+0.5
    ind=which(yy==nreads)
    if(length(ind)>0) yy[ind]=yy[ind]-0.5
  
    mu.sample=sum(yy)/sum(nreads)
    p=yy/nreads
    v=p*(1-p)/nreads
    st[i]=sum((p-mu.sample)^2/v)
    stopifnot(st[i]>=0)
  }
  mu.stats[[mu.i]]=st
  mu.i=mu.i+1
  print(mu)
}

#save(mu.stats,mus,file="Q_distribution_nt30_nreads10.Rdata")
#load("Q_distribution_nt30_nreads10.Rdata")


#Compare Bayesian probability and heterogeneity stats:
par.tag="2000-2000_36-12_80-1"
st.cochran=list()
st.pval=list()
st.bay=list()
for(set in 1:4){
  
  if(set==1) load(paste("data/3class_simu_res_nt_30_0_0_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==2) load(paste("data/3class_simu_res_nt_29_0_1_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==3) load(paste("data/3class_simu_res_nt_0_0_30_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==4) load(paste("data/3class_simu_res_nt_0_1_29_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))

  nsets=length(data.list)
  stopifnot(nsets==1000)
  
  st=rep(NA,nsets)
  st.p=rep(NA,nsets)
  bay=rep(NA,nsets)
  for(i in 1:nsets){
    y=data.list[[i]]$y
    nreads=rowSums(y)
    mu=sum(y[,1])/sum(nreads)
    
    ind=which(y[,1]==0)
    if(length(ind)>0) y[ind,]=y[ind,]+matrix(rep(c(0.5,-0.5),length(ind)),byrow=TRUE,ncol=2)
    ind=which(y[,2]==0)
    if(length(ind)>0) y[ind,]=y[ind,]+matrix(rep(c(-0.5,0.5),length(ind)),byrow=TRUE,ncol=2)

    mu.sample=sum(y[,1])/sum(nreads)
    p=y[,1]/nreads
    v=p*(1-p)/nreads
    st[i]=sum((p-mu.sample)^2/v)
    stopifnot(st[i]>=0)
    mu=min(c(mu,1-mu))
    k=which.min(abs(mu-mus))
    st.p[i]=sum(mu.stats[[k]]>st[i])/length(mu.stats[[k]])
    bay[i]=as.numeric(res.list[[i]]$config.posteriors["HET0"]+res.list[[i]]$config.posteriors["HET1"])
    
  }
  print(summary(st))
  print(summary(st.p))
  print(summary(bay))
  st.cochran[[set]]=st
  st.pval[[set]]=st.p
  st.bay[[set]]=bay
}

#Draw ROCs
#pdf(paste("gibbs_Q_ROC_",par.tag,".pdf",sep=""),width=6,height=3,paper="special",pointsize=8)
#postscript(paste("gibbs_Q_ROC_",par.tag,".eps",sep=""),width=6,height=3,paper="special",pointsize=8)

par(mfrow=c(1,2))
par(mar=c(5,5,5,1))
titles=c("SCENARIO 4 vs SCENARIO 1","SCENARIO 7 vs SCENARIO 3")
for(sets in 1:2){
  
  plot(0,xlim=c(0,1),ylim=c(0,1),col="white",xlab="FALSE POSITIVE RATE",ylab="TRUE POSITIVE RATE",main=titles[sets],cex.lab=1.1)
  cochran=c(st.cochran[[(sets-1)*2+1]],st.cochran[[(sets-1)*2+2]])
  pval=c(st.pval[[(sets-1)*2+1]],st.pval[[(sets-1)*2+2]])
  bayesian=c(st.bay[[(sets-1)*2+1]],st.bay[[(sets-1)*2+2]])
  n=length(pval)/2
  
  #Cochran statistic directly
  #fp=cumsum(rev(order(cochran))<=(nsets))/nsets
  #tp=cumsum(rev(order(cochran))>(nsets))/nsets
  #lines(fp,tp,col="black",lty=2)

  #Use pvalue of Cochran statistics
  print(sum(pval<1e-10))
  fp=cumsum(order(pval)<=n)/n
  tp=cumsum(order(pval)>n)/n
  lines(fp,tp,col="black",lty=2,lwd=1)
  pval.concordant=0
  for( i in 1:n){pval.concordant=pval.concordant+sum(pval[i]>pval[(n+1):(2*n)])+0.5*sum(pval[i]==pval[(n+1):(2*n)])}

  fp=cumsum(rev(order(bayesian))<=(nsets))/nsets
  tp=cumsum(rev(order(bayesian))>(nsets))/nsets
  lines(fp,tp,col="black",lty=1,lwd=1)
  bayesian.concordant=0
  for( i in 1:n){bayesian.concordant=bayesian.concordant+sum(bayesian[i]<bayesian[(n+1):(2*n)])+0.5*sum(bayesian[i]==bayesian[(n+1):(2*n)])}

  legend("bottomright",legend=c(paste("GTM (",signif(bayesian.concordant/n^2,3),")",sep=""),paste("Q (",signif(pval.concordant/n^2,3),")",sep="")),lty=c(1,2))
  
}


#dev.off()



###########
FIGURE 3 ENDS.
###########



###########
FIGURE 4 BEGINS.
###########


a0=b0=2000
a1=36;b1=12
a2=80;b2=1

source("ASE_29Dec2014.R")
y=matrix(c(16,2,14,9,9,3,10,7,13,2,9,1,20,24),ncol=2,byrow=TRUE) #col1 for ref allele
row.names(y)=c("ARTTBL","NERVET","THYROID","ADPSBQ", "HRTLV", "MSCLSK" ,"SKINS") 
res=gtm(y=y,pr.beta=c(a0,b0,a1,b1,a2,b2),niter=2000,burnin=10,two.sided=FALSE,pr.p0=0.75,pr.dist=NULL)

#pdf(paste("rs121909115_",a0,"-",b0,"_",a1,"-",b1,"_",a2,"-",b2,".pdf",sep=""),width=4,height=6.5,paper="special",pointsize=10)
#postscript(paste("rs121909115_",a0,"-",b0,"_",a1,"-",b1,"_",a2,"-",b2,".eps",sep=""),width=4,height=6.5,paper="special",pointsize=10)
plot.gtm(res,y,title.text="rs121909115")
#dev.off()
res

#$state.posteriors 
#         NOASE     MODASE       SNGASE      HET0        HET1   TIS_SPE
#1 0.0004417323 0.06737377 1.738804e-11 0.9306219 0.001562586 0.2354155


###########
FIGURE 4 ENDS.
###########




##### Exact calculation of state probabilities for Fig. 4 example data set.





source("ASE_29Dec2014.R")
y=matrix(c(16,2,14,9,9,3,10,7,13,2,9,1,20,24),ncol=2,byrow=TRUE) #col1 for ref allele
pr.beta=c(2000,2000,36,12,80,1) #prior parameters

nt=nrow(y)
lpr=logprior.distance(nt)
logmlk=matrix(NA,ncol=nt+2,nrow=3^nt)
i.logmlk=1

i1<-2^(0:(nt-1))
i2<-2*i1
for(null.set in 0:(2^nt-1)){#any subset can be equal to NULL
  gr=rep(NA,nt)
  gr[(null.set %% i2)>=i1]=0 #these are NULL
  
  ind=which(is.na(gr)) #these are non-NULL groups
  k=length(ind)
  
  if(k>0){#all set partitions among them
    #idea: elem ind[i] belongs to the set labelled s[i]

    j1<-2^(0:(k-1))
    j2<-2*j1

    for(mod.set in 0:(2^k-1)){#any subset can be equal to MODASE
      ind.mod=(mod.set %% j2)>=j1
      gr[ind[ind.mod]]=1 #these are MOD
      gr[ind[!ind.mod]]=2 #these are SNG
      #do calculations for this partition
        #print(gr)
      logmlk[i.logmlk,]=c(gr,logmlk.3.truncated(y,gr,pr.beta=pr.beta,pr.intv=rep(NA,6),two.sided=FALSE,independent=FALSE),nt-max(as.numeric(table(gr))))
      i.logmlk=i.logmlk+1
    }
  }else{#null model, groups=0
      #print(gr)
    logmlk[i.logmlk,]=c(gr,logmlk.3.truncated(y,gr,pr.beta=pr.beta,pr.intv=rep(NA,6),two.sided=FALSE,independent=FALSE),nt-max(as.numeric(table(gr))))
    i.logmlk=i.logmlk+1
  }
}
  
homog.ind=c(nrow(logmlk),2^nt,1)
for(ii in 1:3){stopifnot(all(logmlk[homog.ind[ii],1:nt]==(ii-1)))}
logmlk=logmlk[-homog.ind,]
ind=(rowSums(logmlk[,1:nt]==0)>0)
  
logmlk.t=logmlk[ind,nt+1]+lpr$log.prior[logmlk[ind,nt+2]]
max.const=max(logmlk.t)
logmlk.h0=max.const+log(sum(exp(logmlk.t-max.const)))-lpr$log.sum.prior.h0  

logmlk.t=logmlk[!ind,nt+1]+lpr$log.prior[logmlk[!ind,nt+2]]
max.const=max(logmlk.t)
logmlk.h1=max.const+log(sum(exp(logmlk.t-max.const)))-lpr$log.sum.prior.h1  

ind=(logmlk[,nt+2]==1)
logmlk.t=logmlk[ind,nt+1]
max.const=max(logmlk.t)
logmlk.d1=max.const+log(mean(exp(logmlk.t-max.const)))  

logmlk.0=logmlk.3.truncated(y,gr=rep(0,nt),pr.beta=pr.beta,pr.intv=rep(NA,6),two.sided=FALSE,independent=FALSE)
logmlk.1=logmlk.3.truncated(y,gr=rep(1,nt),pr.beta=pr.beta,pr.intv=rep(NA,6),two.sided=FALSE,independent=FALSE)
logmlk.2=logmlk.3.truncated(y,gr=rep(2,nt),pr.beta=pr.beta,pr.intv=rep(NA,6),two.sided=FALSE,independent=FALSE)

prior.p0=0.75
ps=c(rep(log(prior.p0/3),3),lpr$log.sum.prior.h0,lpr$log.sum.prior.h1)+c(logmlk.0,logmlk.1,logmlk.2,logmlk.h0,logmlk.h1)
ps=exp(ps-max(ps));ps=ps/sum(ps)
ps=c(ps,exp(log(ps[1])+logmlk.d1-logmlk.0+log((1-prior.p0)/(nt-ceiling(nt/3)))-log(prior.p0/3)))
#ps has the posterior probabilities of states

#> ps
#[1] 4.067604e-04 6.203979e-02 1.601143e-11 9.361061e-01 1.447356e-03
#[6] 2.271822e-01
#(Compare to Fig 4. results above.)




###########
SUPPLEMENTARY FIGURES
###########




########
Supp Figure 3 BEGINS.
########

#data is in the files "3class_simu_res_hm_single_nreads_NREADS_2000-2000_36-12_80-1_niter_100_I.Rdata"
#where NREADS is number of reads per tissue (10 or 50) and I is the index of the collection 1,...,50
#Each collection has 200 variants and 10 tissues as given in 'data.list' list.
#Each collection has also been run through GTM ('res.single', a list with 200 entries)
#and through GTM* ('res.hm')


nsets=50
tags=c("10_0_0","0_10_0","5_5_0","0_5_5")
true.prop=c(0.1,0.3,0,0.4,0.2)
init.path="data/"


#pdf("single_hm_1_3_0_4_2.pdf",width=4,height=8,pointsize=8)
par(mfrow=c(2,1))
for(nreads in c(10,50)){
  single.prob=matrix(0,ncol=5,nrow=nsets)
  hm.prob=matrix(0,ncol=5,nrow=nsets)
  for(i in 1:nsets){
    load(paste(init.path,"3class_simu_res_hm_single_nreads_",nreads,"_2000-2000_36-12_80-1_niter_100_",i,".Rdata",sep=""))
    for(j in 1:length(res.single)){
      single.prob[i,]=single.prob[i,]+as.numeric(res.single[[j]]$config.posteriors[1:5])
    }
    single.prob[i,]=single.prob[i,]/length(res.single)
    hm.prob[i,]=as.numeric(res.hm$prop.posteriors[,1])
  }

  plot(0,xlim=c(0.5,5.5),ylim=c(0,0.6),col="white",xlab="STATE",ylab="STATE PROBABILITY",xaxt="n",main=paste("NUMBER OF READS =",nreads))
  axis(1,at=c(1:5),labels=c("NOASE","MODASE","SNGASE","HET0","HET1"))
  for(i in 1:5){
    x.coord=runif(nsets,-0.05,0.05)
    points(i-0.2+x.coord,hm.prob[,i],pch=19,cex=0.4,col="red")
    points(i+0.2+x.coord,single.prob[,i],pch=15,cex=0.4,col="blue")
    arrows(i-0.2,true.prop[i],i+0.2,true.prop[i],code=0)
    if(nreads==10) legend("topright",pch=c(19,15,NA),lty=c(NA,NA,1),col=c("red","blue","black"),legend=c("GTM*","GTM","TRUTH"))
    if(nreads==50) legend("topleft",pch=c(19,15,NA),lty=c(NA,NA,1),col=c("red","blue","black"),legend=c("GTM*","GTM","TRUTH"))
  }
}
#dev.off()

########
Supp Figure 3 ENDS.
########



###########
Supp FIGURE 4 BEGINS.
###########

source("ASE_29Dec2014.R")
#Assess performance on scenarios 1-9 (Table 1):

#Data for each scenario is in file "../simu_res/single/3class_simu_res_nt_N_M_S_nreads_NREADS_2000-2000_36-12_80-1_niter_2000.Rdata"
#where N, M, and S are the number of tissues in each group, and NREADS is the number of reads per tissue (10, 30, 50 or 200)
#That file contains 'data.list', a list of simulated data set and
#res.list, a list of results from GTM (NOTE that what is config.posteriors here has been changed to state.posteriors) 


par.tag="2000-2000_36-12_80-1"
do.png=FALSE
#if(do.png) png("assessing_probabilities_9_sets.png",width=700,height=900)
#if(!do.png) pdf(paste("assessing_probabilities_9_sets_",par.tag,"_4readcounts.pdf",sep=""),paper="special",width=2*3.5,height=7,pointsize=7)

if(do.png) coeff=1 else coeff=0.8
nts=c(5,10,30)

cols=c("white","gray","black","cyan","violet")
fill.cols=c("white","gray","black","cyan","cyan","violet","violet")
legends=c("NOASE","MODASE","SNGASE","HET0","HET0","HET1","HET1")
legends.details=c("(ALL in N)","(ALL in M)","(ALL in S)","(1 in S ; T-1 in N)","(1 in N ; T-1 in S)","(1 in S ; T-1 in M)","(1 in M ; T-1 in S)","(T/2 in N ; T/2 in M)","(T/2 in M ; T/2 in S)")

if(do.png) layout(matrix(c(1:(4*9)),ncol=4,nrow=9,byrow=FALSE),height=c(1.3,rep(1,7),1.7),width=c(1.2,1,1.2,1))
if(!do.png) layout(matrix(c(1:(4*9)),ncol=4,nrow=9,byrow=FALSE),height=c(1.3,rep(1,7),1.9),width=c(1.2,1,1.2,1))

for(round in 1:4){
  nreads=c(10,30,50,200)[round]
  for(scen in c(1:9)){
    res=matrix(NA,nrow=5,ncol=length(nts))
    icol=1
    for(nt in nts){
      if(scen==1) load(paste("data/3class_simu_res_nt_",nt,"_0_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==2) load(paste("data/3class_simu_res_nt_0_",nt,"_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==3) load(paste("data/3class_simu_res_nt_0_0_",nt,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==4) load(paste("data/3class_simu_res_nt_",nt-1,"_0_1_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==5) load(paste("data/3class_simu_res_nt_1_0_",nt-1,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==6) load(paste("data/3class_simu_res_nt_0_",nt-1,"_1_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==7) load(paste("data/3class_simu_res_nt_0_1_",nt-1,"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==8) load(paste("data/3class_simu_res_nt_",ceiling(nt/2),"_",floor(nt/2),"_0_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
      if(scen==9) load(paste("data/3class_simu_res_nt_0_",ceiling(nt/2),"_",floor(nt/2),"_nreads_",nreads,"_",par.tag,"_niter_2000.Rdata",sep=""))
          
      a=logprior.distance(nt)
      prior.odds=c(rep(0.25,3),exp(c(a$log.sum.prior.h0,a$log.sum.prior.h1)))
      prior.odds=prior.odds/(1-prior.odds)
      nsets=length(data.list)
      chosen=rep(0,5)
      for(i in 1:nsets){
        if(round%%2==1) p=as.numeric(res.list[[i]]$config.posteriors)
        if(round%%2==0) p=as.numeric(res.list[[i]]$state.posteriors)
      
        posterior.odds=p/(1-p)
        chosen=chosen+p[1:5]
      }
      res[,icol]=chosen/nsets
      icol=icol+1
    }
    ord=rev(order(res[,1]))
    if(scen==9){
      if(round%%2==1) {
        par(mar=c(8,4,3,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,xlab="",cex.lab=1.5,horiz=TRUE)
        if(do.png) {
          text(0.5,-1.2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.5)
          legend(0.1,-1.5,legend=c("NOASE"),fill=cols[1],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.4,-1.5,legend=c("MODASE"),fill=cols[2],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.7,-1.5,legend=c("SNGASE"),fill=cols[3],border="black",xpd=TRUE,bty="n",cex=1.5)
        }
        if(!do.png){
          text(0.5,-2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.4)
          legend(-0.2,-2.4,legend=c("NOASE"),fill=cols[1],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.2,-2.4,legend=c("MODASE"),fill=cols[2],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.6,-2.4,legend=c("SNGASE"),fill=cols[3],border="black",xpd=TRUE,bty="n",cex=1.4)
      
        }
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
      }else {
        par(mar=c(8,1,3,2))        
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,xlab="",cex.lab=1.5,horiz=TRUE)
        if(do.png) {
          text(0.5,-1.2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.5)
          legend(0.0,-1.5,legend=c("HET0"),fill=cols[4],border="black",xpd=TRUE,bty="n",cex=1.5)
          legend(0.3,-1.5,legend=c("HET1"),fill=cols[5],border="black",xpd=TRUE,bty="n",cex=1.5)
        }
        if(!do.png){
          text(0.5,-2,paste("STATE PROBABILITY"),xpd=TRUE,cex=1.4)
          legend(0.0,-2.4,legend=c("HET0"),fill=cols[4],border="black",xpd=TRUE,bty="n",cex=1.4)
          legend(0.4,-2.4,legend=c("HET1"),fill=cols[5],border="black",xpd=TRUE,bty="n",cex=1.4)
        }
      }
      axis(1,at=c(0,0.25,0.5,0.75,1),cex.axis=1.3*coeff)
    }
    if(scen>1 & scen < 9){
      if(round%%2==1) {
        par(mar=c(0.1,4,3,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
      }else{
        par(mar=c(0.1,1,3,2))
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,horiz=TRUE)
      }
    }
    if(scen==1){
      if(round%%2==1) {
        par(mar=c(0.1,4,6,2))
        b.plot=barplot(res[ord,],col=cols[ord],xaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(-0.07,b.plot,nts,xpd=TRUE,cex=1.3)
        text(0.5,6,paste("Number of reads =",nreads),xpd=TRUE,cex=1.7)
      }else{
        par(mar=c(0.1,1,6,2))
        barplot(res[ord,],col=cols[ord],xaxt="n",yaxt="n",cex.names=1.3*coeff,horiz=TRUE)
        text(0.5,6,paste("Number of reads =",nreads),xpd=TRUE,cex=1.7)
      }
      
    }
    if(round%%2==1){
      if(do.png) text(0.65,4.2,paste("SCENARIO",scen),xpd=TRUE,cex=1.5)
      if(!do.png) text(0.65,4.2,paste("SCENARIO",scen),xpd=TRUE,cex=1.2)
    }else{
      if(do.png ) text(0.3,4.2,legends.details[scen],xpd=TRUE,cex=1.5)
      if(!do.png) text(0.5,4.2,legends.details[scen],xpd=TRUE,cex=1.2)
    }
    print(res)
  }
}


#dev.off()


###########
Supp FIGURE 4 ENDS.
###########



########
Supp Figure 5 BEGINS
########

#See Figure 3 above for how to create the distributions of Q-statistics that is loaded in here: 
load("Q_distribution_nt30_nreads10.Rdata")


#Compare Bayesian probability and heterogeneity stats:
par.tag="2000-2000_36-12_80-1"
st.pval=list()
st.bay=list()
for(set in 1:6){
  
  if(set==1) load(paste("data/3class_simu_res_nt_30_0_0_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==2) load(paste("data/3class_simu_res_nt_29_0_1_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==3) load(paste("data/3class_simu_res_nt_0_0_30_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==4) load(paste("data/3class_simu_res_nt_0_1_29_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==5) load(paste("data/3class_simu_res_nt_0_30_0_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  if(set==6) load(paste("data/3class_simu_res_nt_0_1_29_nreads_10_",par.tag,"_niter_2000.Rdata",sep=""))
  
  nsets=length(data.list)
  stopifnot(nsets==1000)
  
  st=rep(NA,nsets)
  st.p=rep(NA,nsets)
  bay=rep(NA,nsets)
  for(i in 1:nsets){
    y=data.list[[i]]$y
    nreads=rowSums(y)
    mu=sum(y[,1])/sum(nreads)
    
    ind=which(y[,1]==0)
    if(length(ind)>0) y[ind,]=y[ind,]+matrix(rep(c(0.5,-0.5),length(ind)),byrow=TRUE,ncol=2)
    ind=which(y[,2]==0)
    if(length(ind)>0) y[ind,]=y[ind,]+matrix(rep(c(-0.5,0.5),length(ind)),byrow=TRUE,ncol=2)

    mu.sample=sum(y[,1])/sum(nreads)
    p=y[,1]/nreads
    v=p*(1-p)/nreads
    st[i]=sum((p-mu.sample)^2/v)
    stopifnot(st[i]>=0)
    mu=min(c(mu,1-mu))
    k=which.min(abs(mu-mus))
    st.p[i]=sum(mu.stats[[k]]>st[i])/length(mu.stats[[k]])
    bay[i]=as.numeric(res.list[[i]]$config.posteriors["HET0"]+res.list[[i]]$config.posteriors["HET1"])
    #bay[i]=as.numeric(res.list[[i]]$config.posteriors["HET1"])
    
    
  }
  print(summary(st))
  print(summary(st.p))
  print(summary(bay))
  st.pval[[set]]=st.p
  st.bay[[set]]=bay
}

#Draw ROCs
#postscript(paste("gibbs_Q_ROC_",par.tag,"_supp.eps",sep=""),width=9,height=15,paper="special",pointsize=8)
#pdf(paste("gibbs_Q_ROC_",par.tag,"_supp.pdf",sep=""),width=9,height=15,paper="special",pointsize=8)

par(mfcol=c(5,3))
null.lab=c("SCENARIO 1","SCENARIO 3","SCENARIO 2")
alt.lab=c("SCENARIO 4","SCENARIO 7","SCENARIO 7")
titles=c("SCENARIO 4 vs SCENARIO 1","SCENARIO 7 vs SCENARIO 3","SCENARIO 7 vs SCENARIO 2")
for(sets in 1:3){
  
  par(mar=c(5,5,5,1))
  plot(0,xlim=c(0,1),ylim=c(0,1),col="white",xlab="FALSE POSITIVE RATE",ylab="TRUE POSITIVE RATE",main=titles[sets],cex.lab=1.3,cex.main=1.3,cex.axis=1.3)
  pval=c(st.pval[[(sets-1)*2+1]],st.pval[[(sets-1)*2+2]])
  bayesian=c(st.bay[[(sets-1)*2+1]],st.bay[[(sets-1)*2+2]])
  n=length(pval)/2
  
  #Use pvalue of Cochran statistics
  print(sum(pval<1e-10))
  fp=cumsum(order(pval)<=n)/n
  tp=cumsum(order(pval)>n)/n
  lines(fp,tp,col="black",lty=2,lwd=1)
  pval.concordant=0
  for( i in 1:n){pval.concordant=pval.concordant+sum(pval[i]>pval[(n+1):(2*n)])+0.5*sum(pval[i]==pval[(n+1):(2*n)])}

  fp=cumsum(rev(order(bayesian))<=n)/n
  tp=cumsum(rev(order(bayesian))>n)/n
  lines(fp,tp,col="black",lty=1,lwd=1)
  bayesian.concordant=0
  for( i in 1:n){bayesian.concordant=bayesian.concordant+sum(bayesian[i]<bayesian[(n+1):(2*n)])+0.5*sum(bayesian[i]==bayesian[(n+1):(2*n)])}

  legend("bottomright",legend=c(paste("GTM (",signif(bayesian.concordant/n^2,3),")",sep=""),paste("Q (",signif(pval.concordant/n^2,3),")",sep="")),lty=c(1,2),cex=1.4)

  #plot(0,col="white",xlim=c(0,1),ylim=c(0,10),xlab="HET prob",ylab="density")
  #lines(density(bayesian[1:n],n=50,adj=2),col="black")
  #lines(density(bayesian[(n+1):(2*n)],n=50,adj=2),col="red")
  
  #hist(pval[(n+1):(2*n)],breaks=20)

  #QQ-plot of null vs alt using ranks so that both methods fit the same plot
  #ranks=rank(bayesian)
  #xs=ranks[1:n][order(bayesian[1:n])]
  #ys=ranks[(n+1):(2*n)][order(bayesian[(n+1):(2*n)])]
  #plot(xs,ys,xlim=c(0,2*n),ylim=c(0,2*n),t="l")
  #abline(0,1)
  #ranks=rank(-pval)
  #xs=ranks[1:n][order(-pval[1:n])]
  #ys=ranks[(n+1):(2*n)][order(-pval[(n+1):(2*n)])]
  #lines(xs,ys,col="red")
  
  qqplot(bayesian[1:n],bayesian[(n+1):(2*n)],xlab=paste(null.lab[sets],"(HET proba)"),ylab=paste(alt.lab[sets],"(HET proba)"),main="QQ-PLOT, GTM",xlim=c(0,1),ylim=c(0,1),t="b",pch="+",cex.lab=1.3,cex.main=1.3,cex.axis=1.3)
  abline(0,1)
  qqplot(-log10(pval[1:n]),-log10(pval[(n+1):(2*n)]),xlab=paste(null.lab[sets],"(-log10(pval))"),ylab=paste(alt.lab[sets],"(-log10(pval))"),main="QQ-PLOT, Q-STATISTIC",xlim=c(0,6),ylim=c(0,6),t="b",pch="+",cex.lab=1.3,cex.main=1.3,cex.axis=1.3)
  abline(0,1)
  
  xs=rank(bayesian,ties="average")[1:n]
  ys=rank(-pval,ties="average")[1:n]
  print(mean(xs>ys))
  plot(xs,ys,col=c(rep("black",n)),xlab="GTM rank",ylab="Q-STAT rank",main=null.lab[sets],cex=0.8,pch="+",cex.lab=1.3,cex.main=1.3,cex.axis=1.3)
  abline(0,1)
  xs=rank(bayesian,ties="average")[(n+1):(2*n)]
  ys=rank(-pval,ties="average")[(n+1):(2*n)]
  print(mean(xs>ys))
  plot(xs,ys,col=rep("black",n),xlab="GTM rank",ylab="Q-STAT rank",main=alt.lab[sets],cex=0.8,pch="+",cex.lab=1.3,cex.main=1.3,cex.axis=1.3)
  abline(0,1)
}


#dev.off()

########
Supp Figure 5 ENDS
########

